/****************************************************************************/
/* Test DMX512 + MOD-24LR / nrf24L01                                        */
/****************************************************************************/
/* LEnsE / Julien VILLEMEJANE          Rémi GAROT--JACQUEY / Elisa JARRY    */
/****************************************************************************/
/*  Brochage                                                                */
/*     DMX LED output                                                       */
/*     |    Ch 1-7  : Intensity R G B W Strob Effect                        */
/*     |    Ch 11-17: Intensity R G B W Strob Effect                        */
/*     MIDI piano input                                                     */
/*     D3 Sound Output Channel 1                                            */
/*     D5 Sound Output Channel 2                                            */
/****************************************************************************/
/*  Test réalisé sur Nucléo-L476RG                                          */
/****************************************************************************/
#include    "mbed.h"
#include    "DMX_MIDI.h"
#include    "projecteurs.h"
#include    "controleurs.h"
#include    <cstdint>
#include    <stdbool.h>

#define SAMPLES     512

Serial          debug_pc(USBTX, USBRX);
DigitalOut      debug_out(D13);

//Clavier
PwmOut          speaker1(D3);
PwmOut          speaker2(D5);

//initialisation clavier
void modeclavier(void);

char channel1UsedBy=0x00; char channel2UsedBy=0x00; //On initialise les différents canaux sonores comme étant libres
int freq; int debut_clavier = (int)0x2C; //On définit la tonalité du clavier
float listefreq[]={130.8,138.6,146.8,155.6,164.8,174.6,185,196,207.6,220,233,246.9,262,277,294.6,311.2,330,349,370,392,415,440,466,494,523,554,589,622,660,698,740,784,831,880,932,968,1046};
int liste_color[]={175,50,60,180,52,29,202,71,19,174,134,8,75,139,40,41,141,57,49,95,105,53,100,164,52,70,123,56,63,118,108,55,73156,34,107,207,60,55,230,64,35,235,76,13,205,155,3,105,171,30,30,158,60,40,111,108,50,115,207,59,82,178,60,62,154,115,53,101,183,35,135,236,61,68,234,65,40230,83,10,244,187,4,159,216,17,29,154,57,35,125,107,55,123,210,54,89,179,62,67,159,115,49,119,219,37,150,231,59,68};
bool isChannel1Old=true;
int index;
bool modeClavierOn = true;

// Main
int main(){
    //Initialisation
    debug_pc.baud(9600); debug_pc.printf("Clavier BG\r\n");
    initDMX(); initMIDI();
    speaker1.write(0); speaker2.write(0); //On ne joue aucun son au début
    dmx_data[0] = 255; dmx_data[1] = 0; dmx_data[2] = 0; dmx_data[3] = 0; dmx_data[4] = 0; dmx_data[5] = 0; dmx_data[6] = 0; //Initialisation des DMX des 1ere LED
    dmx_data[10] = 255; dmx_data[11] = 0; dmx_data[12] = 0; dmx_data[13] = 0; dmx_data[14] = 0; dmx_data[15] = 0; dmx_data[16] = 0; // Initialisation des DMX des 2emes LED
    updateDMX();
    modeclavier();
    while(1){ 
    }
            
}

void modeclavier(void){
    while(1){
        if(isNoteMIDIdetected()){
            playNoteMIDI2(note_data, velocity_data);
        // Si on enfonce sur la note, on va mettre le son et les lumières
            if(velocity_data){
                index=(int)note_data-debut_clavier;
            //Si le 1 est libre OU si on occupe déjà le 1 et le 2, alors on utilise le 1
                if(isChannel1Old){ 
                //On met le son
                    speaker1.write(0);
                    speaker1.period(1.0/listefreq[index]);
                    speaker1.write(0.5);
                //On met la lumière
                    dmx_data[1]=liste_color[3*index];
                    debug_pc.printf("rouge 1\r\n");
                    dmx_data[2]=liste_color[3*index+1];
                    debug_pc.printf("vert 1\r\n");
                    dmx_data[3]=liste_color[3*index+2];
                    debug_pc.printf("bleu 1\r\n");
                //On indique qu'on occupe le channel 1
                    channel1UsedBy=note_data;
                    isChannel1Old=!isChannel1Old;
                    updateDMX();
                }
            //Sinon on force l'utilisation du channel 2
                else{
                //On met le son
                    speaker2.write(0);
                    speaker2.period(1.0/listefreq[index]);
                    speaker2.write(0.5);
                //On met la lumière
                    dmx_data[11]=liste_color[3*index];
                    debug_pc.printf("rouge 2\r\n");
                    dmx_data[12]=liste_color[3*index+1];
                    debug_pc.printf("vert 2\r\n");
                    dmx_data[13]=liste_color[3*index+2];
                    debug_pc.printf("bleu 2\r\n");
                //On indique qu'on occupe le channel 2
                    channel2UsedBy=note_data;
                    isChannel1Old=!isChannel1Old;
                    updateDMX();
                }
            }
        //Si on relève la note, on va enlever le son et les lumières
            else{
            //Si on utilisait le channel 1, c'est lui qu'on libère
                if(channel1UsedBy==note_data){
                    //On reset le son
                    speaker1.write(0.0);
                    //On reset les lumières
                    dmx_data[1]=0;
                    dmx_data[2]=0;
                    dmx_data[3]=0;
                    //On indique que le channel est libre
                    channel1UsedBy=0x00;
                    isChannel1Old=true;
                    updateDMX();
                }
                //Si on utilisait le channel 2, c'est lui qu'on libère
                else{
                    //On reset le son
                    speaker2.write(0.0);
                    //On reset les lumières
                    dmx_data[11]=0;
                    dmx_data[12]=0;
                    dmx_data[13]=0;
                    //On indique que le channel est libre
                    channel2UsedBy=0x00;
                    isChannel1Old=false;
                    updateDMX();
                }
            }
            resetNoteMIDI();
        }
    }
}